SystemOrganization addCategory: #'Seaside-Tests-Component-Stubs'!
SystemOrganization addCategory: #'Seaside-Tests-Component-Tests'!

TestCase subclass: #WAComponentTest
	instanceVariableNames: 'log root decoration1 decoration2 child1 child2 child3 child4'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Component-Tests'!

!WAComponentTest methodsFor: 'private' stamp: 'lr 10/22/2008 22:52'!
assertLog: anArray
	self
		assert: log asArray = anArray
		description: 'Expected ' , anArray printString , ', but got ' , log asArray printString.
	log := OrderedCollection new! !

!WAComponentTest methodsFor: 'private' stamp: 'jf 1/23/2009 12:02'!
assertRenders: aString
	| guide |
	guide := self renderingGuide start: root.
	self assert: guide client renderContext document stream contents = aString! !

!WAComponentTest methodsFor: 'private' stamp: 'lr 10/22/2008 22:47'!
log: aString
	log add: aString! !

!WAComponentTest methodsFor: 'accessing' stamp: 'lr 2/4/2009 08:02'!
renderContext
	^ WARenderContext new
		actionUrl: WAUrl new;
		document: (WAHtmlDocument on: (WriteStream on: String new));
		yourself! !

!WAComponentTest methodsFor: 'accessing' stamp: 'jf 1/23/2009 12:04'!
renderingGuide
	| context guide |
	context := self renderContext.
	guide := WARenderingGuide client: (WARenderVisitor context: context).
	context visitor: guide.
	^ guide! !

!WAComponentTest methodsFor: 'running' stamp: 'jf 9/4/2009 16:37'!
setUp
	super setUp.
	log := OrderedCollection new.
	root := WAComponentStub new.
	decoration1 := WADecorationStub new.
	decoration2 := WADecorationStub new.
	child1 := WAComponentStub new.
	child2 := WAComponentStub new.
	child3 := WAComponentStub new.
	child4 := WAComponentStub new! !

!WAComponentTest methodsFor: 'tests' stamp: 'lr 11/4/2008 10:09'!
testActiveComponent

	" single root component "
	self assert: root activeComponent = root.

	" add two children "
	root onChildren: [ Array with: child1 with: child2 ].
	self assert: root activeComponent = root.
	self assert: child1 activeComponent = child1.
	self assert: child2 activeComponent = child2.

	" add two decorations "
	root addDecoration: decoration1; addDecoration: decoration2.
	self assert: root activeComponent = root.
	self assert: child1 activeComponent = child1.
	self assert: child2 activeComponent = child2.
	
	" delegate to another component "
	child2 show: child3.
	self assert: root activeComponent = root.
	self assert: child1 activeComponent = child1.
	self assert: child2 activeComponent = child3.
	self assert: child3 activeComponent = child3.
	
	" delegate to yet another component "
	child3 show: child4.
	self assert: root activeComponent = root.
	self assert: child1 activeComponent = child1.
	self assert: child2 activeComponent = child4.
	self assert: child3 activeComponent = child4.
	self assert: child4 activeComponent = child4
	! !

!WAComponentTest methodsFor: 'tests' stamp: 'lr 10/7/2009 22:57'!
testAddDecoration
	self assert: (root addDecoration: decoration1) = decoration1.
	self assert: (root decorations size = 1).
	self assert: (root decorations first = decoration1).
	
	self assert: (root addDecoration: decoration2) = decoration2.
	self assert: (root decorations size = 2).
	self assert: (root decorations first = decoration2).
	self assert: (root decorations last = decoration1)! !

!WAComponentTest methodsFor: 'tests' stamp: 'lr 7/25/2011 18:59'!
testChildrenNotCollectionError
	"for background see: http://code.google.com/p/seaside/issues/detail?id=586"
	root onChildren: [ ^ Array with: child4 ].
	child1 onChildren: [ ^ self ].
	child2 onChildren: [ ^ child4 ].
	
	self shouldnt: [ root children ] raise: WAChildrenNotCollectionError.

	self should: [ child1 children ] 
		raise: WAChildrenNotCollectionError
		description: 'Testing WAPresenter  >> #childrenDo: with #children return ^self'.

	self should: [ child2 children ] 
		raise: WAChildrenNotCollectionError
		description: 'Testing WAPresenter  >> #childrenDo with #children forgetting to return an array:'.! !

!WAComponentTest methodsFor: 'tests' stamp: 'jf 9/4/2009 16:36'!
testDecorations
	self assert: root decorations isEmpty.
	root show: child1.
	self assert: root decorations size = 1.
	root addDecoration: decoration1.
	self assert: root decorations size = 2
! !

!WAComponentTest methodsFor: 'tests' stamp: 'jf 9/4/2009 16:36'!
testDelegations
	self assert: root delegations isEmpty.
	root show: child1.
	self assert: root delegations size = 1.
	root addDecoration: decoration1.
	self assert: root delegations size = 1! !

!WAComponentTest methodsFor: 'tests' stamp: 'lr 11/4/2008 10:12'!
testHome
	root show: child1.
	child1 show: child2.
	self assert: root activeComponent = child2.
	self assert: child1 activeComponent = child2.
	
	root home.
	self assert: root activeComponent = root.
	self assert: child1 activeComponent = child2.
	self assert: child2 activeComponent = child2.
	
	child1 home.
	self assert: root activeComponent = root.
	self assert: child1 activeComponent = child1.
	self assert: child2 activeComponent = child2.
	
	child2 home.
	self assert: root activeComponent = root.
	self assert: child1 activeComponent = child1.
	self assert: child2 activeComponent = child2! !

!WAComponentTest methodsFor: 'tests' stamp: 'jf 11/3/2008 16:19'!
testInitialRequest
	| request guide |
	request := WARequest new.

	root onInitialRequest: [ :each |
		self assert: each = request.
		self log: 'root' ].
	child1 onInitialRequest: [ :each |
		self assert: each = request.
		self log: 'child1' ].
	child2 onInitialRequest: [ :each |
		self assert: each = request.
		self log: 'child2' ].
	child3 onInitialRequest: [ :each |
		self assert: each = request.
		self log: 'child3' ].
	decoration1 onInitialRequest: [ :each |
		self assert: each = request.
		self log: 'decoration1' ].
	decoration2 onInitialRequest: [ :each |
		self assert: each = request.
		self log: 'decoration2' ].

	guide := WAAllPresenterGuide client: (WAInitialRequestVisitor request: request).

	" single root component "
	guide start: root.
	self assertLog: #('root').

	" add two children "
	root onChildren: [ Array with: child1 with: child2 ].
	guide start: root.
	self assertLog: #('root' 'child1' 'child2').

	" add two decorations "
	root addDecoration: decoration1; addDecoration: decoration2.
	guide start: root.
	self assertLog: #('decoration2' 'decoration1' 'root' 'child1' 'child2').
	
	
	" delegate to another component "
	child2 show: child3.
	guide start: root.
	self assertLog: #('decoration2' 'decoration1' 'root' 'child1' 'child3' 'child2')! !

!WAComponentTest methodsFor: 'tests' stamp: 'lr 10/7/2009 22:56'!
testRemoveDecoration
	root addDecoration: decoration1; addDecoration: decoration2.
	self assert: (root removeDecoration: decoration2) = decoration2.
	self assert: (root decorations size = 1).
	self assert: (root decorations first = decoration1).
	
	root addDecoration: decoration2.
	self assert: (root removeDecoration: decoration1) = decoration1.
	self assert: (root decorations size = 1).
	self assert: (root decorations first = decoration2).
	
	self assert: (root removeDecoration: decoration1) isNil.
	self assert: (root decorations size = 1).
	self assert: (root decorations first = decoration2).

	self assert: (root removeDecoration: decoration2) = decoration2.
	self assert: (root decorations isEmpty).
		
	self assert: (root removeDecoration: decoration2) isNil! !

!WAComponentTest methodsFor: 'tests' stamp: 'jok 8/28/2011 07:50'!
testRemoveDelegation
	self assert: root delegations isEmpty.
	root show: child1.
	root show: child2.
	self assert: root delegations size = 2.
	root removeDelegation.
	self assert: root delegations size = 1.
	root removeDelegation.
	self assert: root delegations isEmpty.
	root removeDelegation.
	self assert: root delegations isEmpty! !

!WAComponentTest methodsFor: 'tests-rendering' stamp: 'lr 11/5/2008 17:10'!
testRenderingChildren
	root onChildren: [ Array with: child1 with: child2 ].
	child1 onRender: [ :html | html text: '1' ].
	child2 onRender: [ :html | html text: '2' ].
	self assertRenders: '12'.
	
	root onRender: [ :html | html text: 'a'; render: child1; text: 'b'; render: child2; text: 'c' ].
	self assertRenders: 'a1b2c'.
	
	root onChildren: [ Array with: child2 with: child1 ].
	self assertRenders: 'a1b2c'.

	root onRender: [ :html | html text: 'a'; render: child2; text: 'b'; render: child1; text: 'c' ].
	self assertRenders: 'a2b1c'! !

!WAComponentTest methodsFor: 'tests-rendering' stamp: 'lr 11/5/2008 17:10'!
testRenderingDecoration
	root onRender: [ :html | html text: 'r' ].
	root addDecoration: decoration1.
	
	decoration1 onRender: [ :html :cc | html text: '1'. cc value ].
	self assertRenders: '1r'.
	
	decoration1 onRender: [ :html :cc | cc value. html text: '1' ].
	self assertRenders: 'r1'.
	
	decoration1 onRender: [ :html :cc | html text: '1' ].
	self assertRenders: '1'! !

!WAComponentTest methodsFor: 'tests-rendering' stamp: 'lr 11/5/2008 17:11'!
testRenderingDelegation
	root onRender: [ :html | html text: 'r' ].
	child1 onRender: [ :html | html text: '1' ].
	child2 onRender: [ :html | html text: '2' ].
	child3 onRender: [ :html | html text: '3' ].
	
	root show: child1.
	self assertRenders: '1'.
	
	child1 show: child2.
	self assertRenders: '2'.
	
	child2 answer.
	self assertRenders: '1'.
	
	root show: child3.
	self assertRenders: '3'.
	
	root home.
	self assertRenders: 'r'
	
	
	! !

!WAComponentTest methodsFor: 'tests' stamp: 'jf 11/3/2008 16:16'!
testUpdateRoot
	| htmlRoot guide |
	htmlRoot := WAHtmlRoot new.

	root onUpdateRoot: [ :each |
		self assert: each = htmlRoot.
		self log: 'root' ].
	child1 onUpdateRoot: [ :each |
		self assert: each = htmlRoot.
		self log: 'child1' ].
	child2 onUpdateRoot: [ :each |
		self assert: each = htmlRoot.
		self log: 'child2' ].
	child3 onUpdateRoot: [ :each |
		self assert: each = htmlRoot.
		self log: 'child3' ].
	decoration1 onUpdateRoot: [ :each |
		self assert: each = htmlRoot.
		self log: 'decoration1' ].
	decoration2 onUpdateRoot: [ :each |
		self assert: each = htmlRoot.
		self log: 'decoration2' ].

	guide := WAVisiblePresenterGuide client: (WAUpdateRootVisitor root: htmlRoot).

	" single root component "
	guide start: root.
	self assertLog: #('root').

	" add two children "
	root onChildren: [ Array with: child1 with: child2 ].
	guide start: root.
	self assertLog: #('root' 'child1' 'child2').

	" add two decorations "
	root addDecoration: decoration1; addDecoration: decoration2.
	guide start: root.
	self assertLog: #('decoration2' 'decoration1' 'root' 'child1' 'child2').
	
	" delegate to another component "
	child2 show: child3.
	guide start: root.
	self assertLog: #('decoration2' 'decoration1' 'root' 'child1' 'child3')! !

!WAComponentTest methodsFor: 'tests' stamp: 'jf 11/3/2008 16:13'!
testUpdateStates
	| snapshot guide |
	snapshot := WASnapshot new.

	root onUpdateStates: [ :each |
		self assert: each = snapshot.
		self log: 'root' ].
	child1 onUpdateStates: [ :each |
		self assert: each = snapshot.
		self log: 'child1' ].
	child2 onUpdateStates: [ :each |
		self assert: each = snapshot.
		self log: 'child2' ].
	child3 onUpdateStates: [ :each |
		self assert: each = snapshot.
		self log: 'child3' ].
	decoration1 onUpdateStates: [ :each |
		self assert: each = snapshot.
		self log: 'decoration1' ].
	decoration2 onUpdateStates: [ :each |
		self assert: each = snapshot.
		self log: 'decoration2' ].

	guide := WAAllPresenterGuide client: (WAUpdateStatesVisitor snapshot: snapshot).

	" single root component "
	guide start: root.
	self assertLog: #('root').

	" add two children "
	root onChildren: [ Array with: child1 with: child2 ].
	guide start: root.
	self assertLog: #('root' 'child1' 'child2').

	" add two decorations "
	root addDecoration: decoration1; addDecoration: decoration2.
	guide start: root.
	self assertLog: #('decoration2' 'decoration1' 'root' 'child1' 'child2').
	
	" delegate to another component "
	child2 show: child3.
	guide start: root.
	self assertLog: #('decoration2' 'decoration1' 'root' 'child1' 'child3' 'child2')! !

!WAComponentTest methodsFor: 'tests' stamp: 'jf 11/3/2008 16:10'!
testUpdateUrl
	| url guide |
	url := WAUrl new.

	root onUpdateUrl: [ :each |
		self assert: each = url.
		self log: 'root' ].
	child1 onUpdateUrl: [ :each |
		self assert: each = url.
		self log: 'child1' ].
	child2 onUpdateUrl: [ :each |
		self assert: each = url.
		self log: 'child2' ].
	child3 onUpdateUrl: [ :each |
		self assert: each = url.
		self log: 'child3' ].
	decoration1 onUpdateUrl: [ :each |
		self assert: each = url.
		self log: 'decoration1' ].
	decoration2 onUpdateUrl: [ :each |
		self assert: each = url.
		self log: 'decoration2' ].

	guide := WAVisiblePresenterGuide client: (WAUpdateUrlVisitor url: url).

	" single root component "
	guide start: root.
	self assertLog: #('root').

	" add two children "
	root onChildren: [ Array with: child1 with: child2 ].
	guide start: root.
	self assertLog: #('root' 'child1' 'child2').

	" add two decorations "
	root addDecoration: decoration1; addDecoration: decoration2.
	guide start: root.
	self assertLog: #('decoration2' 'decoration1' 'root' 'child1' 'child2').
	
	" delegate to another component "
	child2 show: child3.
	guide start: root.
	self assertLog: #('decoration2' 'decoration1' 'root' 'child1' 'child3')! !

WADecoration subclass: #WADecorationStub
	instanceVariableNames: 'initialRequestBlock updateRootBlock updateUrlBlock updateStatesBlock renderBlock childrenBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Component-Stubs'!

!WADecorationStub methodsFor: 'accessing' stamp: 'lr 11/4/2008 10:00'!
children
	^ childrenBlock isNil
		ifTrue: [ super children ]
		ifFalse: [ super children , childrenBlock value ]! !

!WADecorationStub methodsFor: 'initialization' stamp: 'lr 10/22/2008 22:34'!
initialRequest: aRequest
	super initialRequest: aRequest.
	initialRequestBlock isNil
		ifFalse: [ initialRequestBlock value: aRequest ]! !

!WADecorationStub methodsFor: 'events' stamp: 'lr 11/4/2008 10:00'!
onChildren: aBlock
	childrenBlock := aBlock! !

!WADecorationStub methodsFor: 'events' stamp: 'lr 10/22/2008 22:34'!
onInitialRequest: aBlock
	initialRequestBlock := aBlock! !

!WADecorationStub methodsFor: 'events' stamp: 'lr 10/22/2008 22:34'!
onRender: aBlock
	renderBlock := aBlock! !

!WADecorationStub methodsFor: 'events' stamp: 'lr 10/22/2008 22:34'!
onUpdateRoot: aBlock
	updateRootBlock := aBlock! !

!WADecorationStub methodsFor: 'events' stamp: 'lr 10/22/2008 22:34'!
onUpdateStates: aBlock
	updateStatesBlock := aBlock! !

!WADecorationStub methodsFor: 'events' stamp: 'lr 10/22/2008 22:34'!
onUpdateUrl: aBlock
	updateUrlBlock := aBlock! !

!WADecorationStub methodsFor: 'rendering' stamp: 'lr 10/22/2008 22:35'!
renderContentOn: html
	renderBlock isNil 
		ifTrue: [ super renderContentOn: html ]
		ifFalse: [ renderBlock value: html value: [ super renderContentOn: html ] ]! !

!WADecorationStub methodsFor: 'updating' stamp: 'lr 9/8/2009 12:40'!
updateRoot: anHtmlRoot
	super updateRoot: anHtmlRoot.
	updateRootBlock isNil
		ifFalse: [ updateRootBlock value: anHtmlRoot ]! !

!WADecorationStub methodsFor: 'updating' stamp: 'lr 10/22/2008 22:34'!
updateStates: aSnapshot
	super updateStates: aSnapshot.
	updateStatesBlock isNil
		ifFalse: [ updateStatesBlock value: aSnapshot ]! !

!WADecorationStub methodsFor: 'updating' stamp: 'lr 10/22/2008 22:34'!
updateUrl: aUrl
	super updateUrl: aUrl.
	updateUrlBlock isNil
		ifFalse: [ updateUrlBlock value: aUrl ]! !

WAComponent subclass: #WAComponentStub
	instanceVariableNames: 'initialRequestBlock updateRootBlock updateUrlBlock updateStatesBlock renderBlock childrenBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-Component-Stubs'!

!WAComponentStub methodsFor: 'accessing' stamp: 'lr 10/22/2008 22:29'!
children
	^ childrenBlock isNil
		ifTrue: [ super children ]
		ifFalse: [ childrenBlock value ]! !

!WAComponentStub methodsFor: 'initialization' stamp: 'lr 10/22/2008 22:24'!
initialRequest: aRequest
	super initialRequest: aRequest.
	initialRequestBlock isNil
		ifFalse: [ initialRequestBlock value: aRequest ]! !

!WAComponentStub methodsFor: 'events' stamp: 'lr 10/22/2008 22:28'!
onChildren: aBlock
	childrenBlock := aBlock! !

!WAComponentStub methodsFor: 'events' stamp: 'lr 10/22/2008 22:28'!
onInitialRequest: aBlock
	initialRequestBlock := aBlock! !

!WAComponentStub methodsFor: 'events' stamp: 'lr 10/22/2008 22:23'!
onRender: aBlock
	renderBlock := aBlock! !

!WAComponentStub methodsFor: 'events' stamp: 'lr 10/22/2008 22:23'!
onUpdateRoot: aBlock
	updateRootBlock := aBlock! !

!WAComponentStub methodsFor: 'events' stamp: 'lr 10/22/2008 22:23'!
onUpdateStates: aBlock
	updateStatesBlock := aBlock! !

!WAComponentStub methodsFor: 'events' stamp: 'lr 10/22/2008 22:23'!
onUpdateUrl: aBlock
	updateUrlBlock := aBlock! !

!WAComponentStub methodsFor: 'rendering' stamp: 'lr 10/22/2008 22:25'!
renderContentOn: html
	renderBlock isNil 
		ifTrue: [ super renderContentOn: html ]
		ifFalse: [ renderBlock value: html ]! !

!WAComponentStub methodsFor: 'updating' stamp: 'lr 9/8/2009 12:40'!
updateRoot: anHtmlRoot
	super updateRoot: anHtmlRoot.
	updateRootBlock isNil
		ifFalse: [ updateRootBlock value: anHtmlRoot ]! !

!WAComponentStub methodsFor: 'updating' stamp: 'lr 10/22/2008 22:27'!
updateStates: aSnapshot
	super updateStates: aSnapshot.
	updateStatesBlock isNil
		ifFalse: [ updateStatesBlock value: aSnapshot ]! !

!WAComponentStub methodsFor: 'updating' stamp: 'lr 10/22/2008 22:28'!
updateUrl: aUrl
	super updateUrl: aUrl.
	updateUrlBlock isNil
		ifFalse: [ updateUrlBlock value: aUrl ]! !

!GRPackage class methodsFor: '*seaside-tests-component' stamp: 'lr 2/17/2010 15:13'!
seasideTestsComponent
	^ self new
		name: 'Seaside-Tests-Component';
		description: 'Unit tests for the package Seaside-Component.';
		addDependency: 'Seaside-Component';
		addDependency: 'Seaside-Tests-Core';
		url: #seasideUrl;
		yourself! !
